[PATCH 1/7] Emit a query when writing to a readonly section and trust_readonly is true

Yao Qi yao@codesourcery.com
Fri Sep 20 02:47:00 GMT 2013


If users write readonly section, such as .text, the contents of the inferior
and of the executable become out of sync.  It is better to emit a query
to ask users to "set trust-readonly-sections off".

(gdb) set trust-readonly-sections on
(gdb) p /x* (char *) 0x080484c1 = 0xcc
Address is read-only and trust-readonly-sections is set to "on".
Set it to "off" and write to a read-only section? (y or n) y
$1 = 0xcc

gdb:

2013-09-20  Yao Qi  <yao@codesourcery.com>

	* target.c (memory_xfer_partial_1): Emit a query if GDB
	writes to a readonly section and 'trust_readonly' is true.

gdb/testsuite:

2013-09-20  Yao Qi  <yao@codesourcery.com>

	* gdb.base/trust-readonly.exp: New.
---
 gdb/target.c                              |   35 +++++++++---
 gdb/testsuite/gdb.base/trust-readonly.exp |   85 +++++++++++++++++++++++++++++
 2 files changed, 111 insertions(+), 9 deletions(-)
 create mode 100644 gdb/testsuite/gdb.base/trust-readonly.exp

diff --git a/gdb/testsuite/gdb.base/trust-readonly.exp b/gdb/testsuite/gdb.base/trust-readonly.exp
new file mode 100644
index 0000000..8627800
--- /dev/null
+++ b/gdb/testsuite/gdb.base/trust-readonly.exp
@@ -0,0 +1,85 @@
+# Copyright 2013 Free Software Foundation, Inc.
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+standard_testfile start.c
+
+if  { [gdb_compile "${srcdir}/${subdir}/${srcfile}" "${binfile}" executable {debug}] != "" } {
+    untested "Could not compile $binfile."
+    return -1
+}
+
+clean_restart ${binfile}
+
+if ![runto_main] then {
+    return -1
+}
+
+set main_address ""
+set test "Extract main address"
+gdb_test_multiple "print *main" $test {
+    -re "\} ($hex) .*$gdb_prompt $" {
+	pass $test
+	set main_address $expect_out(1,string)
+    }
+    -re ".*$gdb_prompt $" {
+	# Unable to extract address of main.  Bail out.
+	fail $test
+	return -1
+    }
+}
+
+set main_contents ""
+set test "Extract main contents"
+gdb_test_multiple "print/x *(char *) $main_address" $test {
+    -re "\\$$decimal = ($hex).*$gdb_prompt $" {
+	pass $test
+	set main_contents $expect_out(1,string)
+    }
+    -re ".*$gdb_prompt $" {
+	# Unable to extract contents of main.  Bail out.
+	fail $test
+	return -1
+    }
+}
+
+# Set option "trust-readonly-sections" off, write one byte and read it
+# back.
+
+gdb_test_no_output "set trust-readonly-sections off" ""
+gdb_test "print /x* (char *) $main_address = 0xcc" "\\$$decimal = 0xcc" \
+    "write to main (off)"
+gdb_test "print /x * (char *) $main_address" "\\$$decimal = 0xcc" \
+    "read from main (off) 1"
+
+gdb_test_no_output "set trust-readonly-sections on" ""
+gdb_test "print /x* (char *) $main_address = 0x11" "\\$$decimal = 0x11" \
+    "write to main (on)" \
+    "Address is read-only and trust-readonly-sections is set to \"on\"\..*y or n. $" \
+    "y"
+
+# Option "trust-readonly-sections" is turned off as answer "y" is
+# entered above.
+gdb_test "show trust-readonly-sections" \
+    "Mode for reading from readonly sections is off\."
+
+gdb_test "print /x * (char *) $main_address" "\\$$decimal = 0x11" \
+    "read from main (off) 2"
+
+# Set option "trust-readonly-sections" on, and test GDB read data from
+# the executable file instead of the inferior.
+
+gdb_test_no_output "set trust-readonly-sections on" ""
+gdb_test "print /x * (char *) $main_address" "\\$$decimal = ${main_contents}" \
+    "read from main (on)"
diff --git a/gdb/target.c b/gdb/target.c
index d55712d..37db36b 100644
--- a/gdb/target.c
+++ b/gdb/target.c
@@ -1471,11 +1471,13 @@ memory_xfer_partial_1 (struct target_ops *ops, enum target_object object,
 	}
     }
 
-  /* Try the executable files, if "trust-readonly-sections" is set.  */
-  if (readbuf != NULL && trust_readonly)
+  /* If "trust-readonly-sections" is set, read from the executable
+     files.  When GDB writes to a readonly section of the inferior,
+     emit a warning that option 'trust-readonly-sections' should be
+     turned off.  */
+  if (trust_readonly)
     {
       struct target_section *secp;
-      struct target_section_table *table;
 
       secp = target_section_by_addr (ops, memaddr);
       if (secp != NULL
@@ -1483,12 +1485,27 @@ memory_xfer_partial_1 (struct target_ops *ops, enum target_object object,
 				     secp->the_bfd_section)
 	      & SEC_READONLY))
 	{
-	  table = target_get_section_table (ops);
-	  return section_table_xfer_memory_partial (readbuf, writebuf,
-						    memaddr, len,
-						    table->sections,
-						    table->sections_end,
-						    NULL);
+
+	  if (readbuf != NULL)
+	    {
+	      struct target_section_table *table;
+
+	      table = target_get_section_table (ops);
+	      return section_table_xfer_memory_partial (readbuf, writebuf,
+							memaddr, len,
+							table->sections,
+							table->sections_end,
+							NULL);
+	    }
+
+	  if (writebuf != NULL)
+	    {
+	      if (query (_("Address is read-only and trust-readonly-sections\
+ is set to \"on\".\nSet it to \"off\" and write to a read-only section? ")))
+		trust_readonly = 0;
+	      else
+		return -1;
+	    }
 	}
     }
 
-- 
1.7.7.6



More information about the Gdb-patches mailing list